import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';
import { OrbitControls } from './jsm/controls/OrbitControls.js';

class App {
    constructor() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        //renderer.outputEncoding = THREE.sRGBEncoding;
        renderer.setPixelRatio(window.devicePixelRatio);
        renderer.autoClear = false;
        domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0x000000);
        scene.fog = new THREE.Fog(0x000000, 1500, 4000);
        this._scene = scene;

        const scene2 = new THREE.Scene();
        scene2.background = new THREE.Color(0x000000);
        scene2.fog = new THREE.Fog(0x000000, 1500, 4000);
        this._scene2 = scene2;

        this._setupCamera();
        //this._setupLight();
        //this._setupPostprocessing();
        //this._setupInteractions();
        this._setupModel();
        this._setupGUI();
        this._setupControls();
        //this._setupHelper();

        window.onresize = this.resize.bind(this);

        this.resize();
        this.render()
    }

    _setupInteractions() {
    }

    _setupPostprocessing() {
        //.
    }

    _setupModel() {
        const imageCanvas = document.createElement("canvas");
        const context = imageCanvas.getContext("2d");
        const imageSize = 128;

        imageCanvas.width = imageCanvas.height = imageSize;
        
        context.fillStyle = "#444";
        context.fillRect(0, 0, imageSize, imageSize);
        context.fillStyle = "#fff";
        context.fillRect(0, 0, imageSize/2, imageSize/2);
        context.fillRect(imageSize/2, imageSize/2, imageSize/2, imageSize/2);

        const textureCanvas = new THREE.CanvasTexture(imageCanvas);
        textureCanvas.repeat.set(1000, 1000);
        textureCanvas.wrapS = textureCanvas.wrapT = THREE.RepeatWrapping;

        const textureCanvas2 = textureCanvas.clone();
        textureCanvas2.magFilter = textureCanvas2.minFilter = THREE.NearestFilter;

        const materialCanvas = new THREE.MeshBasicMaterial({map: textureCanvas});
        const materialCanvas2 = new THREE.MeshBasicMaterial({color:0xffccaa, map: textureCanvas2});

        const geometry = new THREE.PlaneGeometry(100, 100);
        const meshCanvas = new THREE.Mesh(geometry, materialCanvas);
        meshCanvas.rotation.x = -Math.PI / 2;
        meshCanvas.scale.set(1000, 1000, 1000);

        const meshCanvas2 = new THREE.Mesh(geometry, materialCanvas2);
        meshCanvas2.rotation.x = -Math.PI / 2;
        meshCanvas2.scale.set(1000, 1000, 1000);

        const callbackPainting = (texture) => {
            const image = texture.image;
            texturePainting2.image = image;
            texturePainting2.needsUpdate = true;

            this._scene.add(meshCanvas);
            this._scene2.add(meshCanvas2);

            const geometry = new THREE.PlaneGeometry(100, 100);
            const mesh = new THREE.Mesh(geometry, materialPainting);
            const mesh2 = new THREE.Mesh(geometry, materialPainting2);

            const floorHeight = -1.117 * image.height / 2;
            meshCanvas.position.y = meshCanvas2.position.y = floorHeight;

            addPainting(this._scene, mesh);
            addPainting(this._scene2, mesh2);

            function addPainting(zscene, zmesh) {
                zmesh.scale.x = image.width / 100;
                zmesh.scale.y = image.height / 100;
                zscene.add(zmesh);
                const meshFrame = new THREE.Mesh(geometry, new THREE.MeshBasicMaterial({color: 0x000000, side: THREE.DoubleSide}));
                meshFrame.position.z = -10.0;
                meshFrame.scale.x = 1.1 * image.width / 100;
                meshFrame.scale.y = 1.1 * image.height / 100;
                zscene.add(meshFrame);

                const meshShadow = new THREE.Mesh(geometry, new THREE.MeshBasicMaterial({color: 0x000000, opacity: 0.75, transparent: true}));
                meshShadow.position.y = -1.1 * image.height / 2;
                meshShadow.position.z = -1.1 * image.height / 2;
                meshShadow.rotation.x = -Math.PI / 2;
                meshShadow.scale.x = 1.1 * image.width / 100;
                meshShadow.scale.y = 1.1 * image.height / 100;
                zscene.add(meshShadow);
            }
        };
        
        const texturePainting = new THREE.TextureLoader().load("textures/758px-Canestra_di_frutta_(Caravaggio).jpg", callbackPainting);
        const texturePainting2 = new THREE.Texture();
        const materialPainting = new THREE.MeshBasicMaterial({color:0xffffff, map: texturePainting});
        const materialPainting2 = new THREE.MeshBasicMaterial({color: 0xffccaa, map: texturePainting2});

        texturePainting2.minFilter = texturePainting2.magFilter = THREE.NearestFilter;
        texturePainting.minFilter = texturePainting.magFilter = THREE.LinearFilter;
        texturePainting.mapping = THREE.UVMapping;
    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        document.body.appendChild(stats.dom);
        this._stats = stats;
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(35, window.innerWidth/window.innerHeight, 1, 5000);
        camera.position.z = 1500;
        this._camera = camera;
    }

    _setupControls() {
        const controls = new OrbitControls(this._camera, this._renderer.domElement);
        controls.update();
    }

    _setupLight() {
    }

    update() {
        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));

        this._renderer.clear();
        this._renderer.setScissorTest(true);

        const width = window.innerWidth;
        const height = window.innerHeight;

        this._renderer.setScissor(0, 0, width/2-1, height);
        this._renderer.render(this._scene, this._camera);
        
        this._renderer.setScissor(width/2, 0, width/2, height);
        this._renderer.render(this._scene2, this._camera);

        this._renderer.setScissor(false);
        
        this.update();
    }

    resize() {
        const width = window.innerWidth;
        const height = window.innerHeight;

        this._camera.aspect = width / height;
        this._camera.updateProjectionMatrix();
        
        this._renderer.setSize(width, height);
    }
}

window.onload = function () {
    new App();
}